@@ -20,12 +20,13 @@ class File(object):
         self.logical_name = ''
         if type(tree) is dict:
             for k, v in tree.items():
-                self.name = os.path.expandvars(k)
+                self.name = String(os.path.expandvars(k))
                 self.file_type       = v.get('file_type', '')
                 self.is_include_file = v.get('is_include_file', False)
                 self.copyto          = v.get('copyto', '')
+                self.logical_name    = v.get('logical_name', '')
         else:
-            self.name = os.path.expandvars(tree)
+            self.name = String(os.path.expandvars(tree))
             self.is_include_file = False #"FIXME"
 
 class Genparams(dict):
@@ -69,19 +70,24 @@ class Section(object):
     lists   = {}
     dicts   = {}
     def __init__(self, tree):
-        for k, v in tree.items():
+        for k, v in sorted(tree.items()):
             if v is None:
                 continue
             if k in self.members:
                 setattr(self, k, globals()[self.members[k]](v))
             elif k in self.lists:
-                _l = []
+                if k.endswith('_append'):
+                    _k = k[:-7]
+                    _l = getattr(self, _k, [])[:]
+                else:
+                    _k = k
+                    _l = []
                 for _item in v:
                     try:
-                        _l.append(globals()[self.lists[k]](_item))
+                        _l.append(globals()[self.lists[_k]](_item))
                     except TypeError as e:
                         raise SyntaxError("Bad option '{}' in section '{}'".format(_item, k))
-                setattr(self, k, _l)
+                setattr(self, _k, _l)
             elif k in self.dicts:
                 if not isinstance(v, dict):
                     raise SyntaxError("Object in '{}' section must be a dict".format(k))
@@ -94,7 +100,7 @@ class Section(object):
                     setattr(_d[_name], 'name', _name)
                 setattr(self, k, _d)
             else:
-                raise KeyError(k + " in section " + self.__class__.__name__)
+                logger.warn("Unknown item " + k + " in section " + self.__class__.__name__)
 
 class Provider(object):
     def __new__(cls, *args, **kwargs):
@@ -276,7 +282,16 @@ class Core:
         src_files = []
         for fs in self._get_filesets(flags):
             src_files += fs.files
-        return src_files
+        _src_files = []
+        for f in src_files:
+            pf = f.name.parse(flags)
+            if pf:
+                _f = File({pf : {'copyto' : f.copyto,
+                                'file_type' : f.file_type,
+                                'is_include_file' : f.is_include_file,
+                                 'logical_name' : f.logical_name}})
+                _src_files.append(_f)
+        return _src_files
 
     def get_generators(self, flags):
         self._debug("Getting generators for flags {}".format(str(flags)))
@@ -438,9 +453,15 @@ Name:      {}
 Core root: {}
 
 Targets:
-{}
-"""
-        targets = '\n'.join(sorted(self.targets))
+{}"""
+        if self.targets:
+            l = max(len(x) for x in self.targets)
+            targets = ""
+
+            for t in sorted(self.targets):
+                targets += "{} : {}\n".format(t.ljust(l), self.targets[t].description or "<No description>")
+        else:
+            targets = "<No targets>"
         return s.format(str(self.name),
                         str(self.core_root),
                         targets)
@@ -494,7 +515,7 @@ Targets:
 
         for fs in self._parse_list(flags, target.filesets):
             if not fs in self.filesets:
-                raise SyntaxError("Fileset '{}', requested by fileset '{}', was not found".format(fs, target.name))
+                raise SyntaxError("{} : Fileset '{}', requested by target '{}', was not found".format(self.name, fs, target.name))
             filesets.append(self.filesets[fs])
 
         self._debug(" Matched filesets {}".format(target.filesets))
@@ -602,6 +623,9 @@ Target:
     - name : default_tool
       type : String
       desc : Default tool to use unless overridden with ``--tool=``
+    - name : description
+      type : String
+      desc : Description of the target
     - name : hooks
       type : Hooks
       desc : Script hooks to run when target is used
@@ -731,6 +755,8 @@ def _generate_classes(j, base_class):
             for key in _items['lists']:
                 class_members[key['name']] = []
                 class_members['lists'][key['name']] = key['type']
+                class_members[key['name']+'_append'] = []
+                class_members['lists'][key['name']+'_append'] = key['type']
         if 'dicts' in _items:
             class_members['dicts'] = {}
             for key in _items['dicts']:
@@ -816,6 +842,40 @@ Vlnv
 ~~~~~~
 :-separated VLNV (Vendor, Library, Name, Vendor) identifier
 
+List
+~~~~
+
+All keys that are defined as lists have a corresponding *<key>_append* key. The items in the *<key>_append* list is automatically appended to the list of the original key.
+
+**Example** A target contains the following keys
+
+::
+
+    filesets : [fileset_a, fileset_b]
+    filesets_append : [fileset_c, fileset_d]
+
+This will be evaluated to
+
+::
+
+    filesets : [fileset_a, fileset_b, fileset_c, fileset_d]
+
+The main use case for this feature is inheritance between targets with yaml anchors as in the following example where a board-specific implementation and a simulation target inherits from the default target
+
+::
+
+    targets:
+      default: &base
+        filesets: [core]
+    
+      myboard:
+        <<: *base
+        filesets_append: [myboard_files]
+    
+      sim:
+        <<: *base
+        filesets_append: [testbench]
+
 Sections
 --------
 
